import ij.ImagePlus;
import ij.gui.GenericDialog;
import ij.plugin.PlugIn;
import ij.process.ImageProcessor;
import imagescience.image.Axes;
import imagescience.image.ColorImage;
import imagescience.image.Dimensions;
import imagescience.image.Image;
import imagescience.transform.Mirror;
import imagescience.utility.I5DResource;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Panel;
import java.awt.image.ColorModel;

public class TJ_Mirror implements PlugIn {

    private static boolean x = false;
    private static boolean y = false;
    private static boolean z = false;
    private static boolean t = false;
    private static boolean c = false;

    public void run(String arg) {

	if (!TJ.libcheck()) return;
	final ImagePlus imp = TJ.imageplus();
	if (imp == null) return;

	TJ.debug(TJ.name()+" "+TJ.version()+": Mirror");

	GenericDialog gd = new GenericDialog(TJ.name()+": Mirror");
	gd.addCheckbox(" x-mirror input image    ",x);
	gd.addCheckbox(" y-mirror input image    ",y);
	gd.addCheckbox(" z-mirror input image    ",z);
	gd.addCheckbox(" t-mirror input image    ",t);
	gd.addCheckbox(" ch-mirror input image    ",c);
	gd.addPanel(new Panel(),GridBagConstraints.EAST,new Insets(0,0,0,0));
	gd.showDialog();

	if (gd.wasCanceled()) return;

	x = gd.getNextBoolean();
	y = gd.getNextBoolean();
	z = gd.getNextBoolean();
	t = gd.getNextBoolean();
	c = gd.getNextBoolean();

	(new TJMirror()).run(imp,x,y,z,t,c);
    }

}

class TJMirror { // Separate class to make the plugin class independent of imagescience.jar

    void run(final ImagePlus imp,
	     final boolean x,
	     final boolean y,
	     final boolean z,
	     final boolean t,
	     final boolean c) {

	try {
	    final Image img = Image.wrap(imp);
	    final Image newimg = img.duplicate();
	    final Mirror mrr = new Mirror();
	    mrr.debug(TJ_Options.debug);
	    mrr.run(newimg,new Axes(x,y,z,t,c));
	    ImagePlus newimp = newimg.imageplus();
	    newimp.setCalibration(imp.getCalibration().copy());
	    final ImageProcessor ip = imp.getProcessor();
	    final ImageProcessor newip = newimp.getProcessor();
	    final ColorModel cm = ip.getColorModel();
	    newip.setColorModel(cm);
	    newimp.getStack().setColorModel(cm);
	    final double[] minmax = newimg.extrema();
	    if (TJ_Options.adopt) newip.setMinAndMax(ip.getMin(),ip.getMax());
	    else newip.setMinAndMax(minmax[0],minmax[1]);
	    boolean i5dinstalled = false;
	    try { Class.forName("i5d.Image5D"); i5dinstalled = true; } catch (Exception e) { }
	    if (i5dinstalled && !(newimg instanceof ColorImage)) {
		final Dimensions newdims = newimg.dimensions();
		if (newdims.t > 1 || newdims.s > 1) {
		    newimp = I5DResource.convert(newimp,true);
		    if (I5DResource.instance(imp)) {
			final int[][] idx = new int[2][newdims.s];
			if (c) for (int i=0; i<newdims.s; ++i) {
			    idx[0][i] = i + 1;
			    idx[1][i] = newdims.s - i;
			} else for (int i=0; i<newdims.s; ++i)
			    idx[0][i] = idx[1][i] = i + 1;
			I5DResource.transfer(imp,newimp,idx);
			if (!TJ_Options.adopt)
			    I5DResource.minmax(newimp,minmax[0],minmax[1]);
		    }
		}
	    }
	    newimp.changes = TJ_Options.asksave;
	    newimp.show();

	} catch (OutOfMemoryError e) {
	    TJ.error("Not enough memory for this operation");

	} catch (Throwable e) {
	    TJ.error("An unidentified error occurred while running the plugin");

	}
    }

}
